@@ -1,36 +1,12 @@
-/*
- * Copyright 1993-2010 University Corporation for Atmospheric Research/Unidata
- * 
- * Portions of this software were developed by the Unidata Program at the 
- * University Corporation for Atmospheric Research.
- * 
- * Access and use of this software shall impose the following obligations
- * and understandings on the user. The user is granted the right, without
- * any fee or cost, to use, copy, modify, alter, enhance and distribute
- * this software, and any derivative works thereof, and its supporting
- * documentation for any purpose whatsoever, provided that this entire
- * notice appears in all copies of the software, derivative works and
- * supporting documentation.  Further, UCAR requests that the user credit
- * UCAR/Unidata in any publications that result from the use of this
- * software or in any product that includes this software. The names UCAR
- * and/or Unidata, however, may not be used in any advertising or publicity
- * to endorse or promote any products or commercial entity unless specific
- * written permission is obtained from UCAR/Unidata. The user also
- * understands that UCAR/Unidata is not obligated to provide the user with
- * any support, consulting, training or assistance of any kind with regard
- * to the use, operation and performance of this software nor to provide
- * the user with any updates, revisions, new versions or "bug fixes."
- * 
- * THIS SOFTWARE IS PROVIDED BY UCAR/UNIDATA "AS IS" AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL UCAR/UNIDATA BE LIABLE FOR ANY SPECIAL,
- * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
- * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
- * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
- * WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
- */
-/* "$Id: netcdf.h,v 1.1 2010/06/01 15:46:49 ed Exp $" */
+/*! \file
+
+Main header file for the C API.
+
+Copyright 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
+2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 University
+Corporation for Atmospheric Research/Unidata. See \ref copyright file
+for more info.
+*/
 
 #ifndef _NETCDF_
 #define _NETCDF_
@@ -38,7 +14,7 @@
 #include <stddef.h> /* size_t, ptrdiff_t */
 #include <errno.h>  /* netcdf functions sometimes return system errors */
 
-/* The nc_type type is just an int. */
+/*! The nc_type type is just an int. */
 typedef int nc_type;
 
 #if defined(__cplusplus)
@@ -48,39 +24,38 @@ extern "C" {
 /*
  *  The netcdf external data types
  */
-#define	NC_NAT 	        0	/* NAT = 'Not A Type' (c.f. NaN) */
-#define	NC_BYTE         1	/* signed 1 byte integer */
-#define	NC_CHAR 	2	/* ISO/ASCII character */
-#define	NC_SHORT 	3	/* signed 2 byte integer */
-#define	NC_INT 	        4	/* signed 4 byte integer */
-#define NC_LONG         NC_INT  /* deprecated, but required for backward compatibility. */
-#define	NC_FLOAT 	5	/* single precision floating point number */
-#define	NC_DOUBLE 	6	/* double precision floating point number */
-#define	NC_UBYTE 	7	/* unsigned 1 byte int */
-#define	NC_USHORT 	8	/* unsigned 2-byte int */
-#define	NC_UINT 	9	/* unsigned 4-byte int */
-#define	NC_INT64 	10	/* signed 8-byte int */
-#define	NC_UINT64 	11	/* unsigned 8-byte int */
-#define	NC_STRING 	12	/* string */
+#define	NC_NAT 	        0	/**< Not A Type */
+#define	NC_BYTE         1	/**< signed 1 byte integer */
+#define	NC_CHAR 	2	/**< ISO/ASCII character */
+#define	NC_SHORT 	3	/**< signed 2 byte integer */
+#define	NC_INT 	        4	/**< signed 4 byte integer */
+#define NC_LONG         NC_INT  /**< deprecated, but required for backward compatibility. */
+#define	NC_FLOAT 	5	/**< single precision floating point number */
+#define	NC_DOUBLE 	6	/**< double precision floating point number */
+#define	NC_UBYTE 	7	/**< unsigned 1 byte int */
+#define	NC_USHORT 	8	/**< unsigned 2-byte int */
+#define	NC_UINT 	9	/**< unsigned 4-byte int */
+#define	NC_INT64 	10	/**< signed 8-byte int */
+#define	NC_UINT64 	11	/**< unsigned 8-byte int */
+#define	NC_STRING 	12	/**< string */
 
 #define NC_MAX_ATOMIC_TYPE NC_STRING
 
 /* The following are use internally in support of user-defines
  * types. They are also the class returned by nc_inq_user_type. */
-#define	NC_VLEN 	13	/* used internally for vlen types */
-#define	NC_OPAQUE 	14	/* used internally for opaque types */
-#define	NC_ENUM 	15	/* used internally for enum types */
-#define	NC_COMPOUND 	16	/* used internally for compound types */
+#define	NC_VLEN 	13	/**< vlen types */
+#define	NC_OPAQUE 	14	/**< opaque types */
+#define	NC_ENUM 	15	/**< enum types */
+#define	NC_COMPOUND 	16	/**< compound types */
 
 /* Define the first user defined type id (leave some room) */
 #define NC_FIRSTUSERTYPEID 32
 
-/*
- * 	Default fill values, used unless _FillValue attribute is set.
- * These values are stuffed into newly allocated space as appropriate.
- * The hope is that one might use these to notice that a particular datum
- * has not been set.
- */
+/** Default fill value. This is used unless _FillValue attribute
+ * is set.  These values are stuffed into newly allocated space as
+ * appropriate.  The hope is that one might use these to notice that a
+ * particular datum has not been set. */
+/**@{*/
 #define NC_FILL_BYTE	((signed char)-127)
 #define NC_FILL_CHAR	((char)0)
 #define NC_FILL_SHORT	((short)-32767)
@@ -93,14 +68,16 @@ extern "C" {
 #define NC_FILL_INT64   ((long long)-9223372036854775806LL)
 #define NC_FILL_UINT64  ((unsigned long long)18446744073709551614ULL)
 #define NC_FILL_STRING  ""
+/**@}*/
 
-/* These represent the max and min values that can be stored in a
- * netCDF file for their associated types. Recall that a C compiler
- * may define int to be any length it wants, but a NC_INT is *always*
- * a 4 byte signed int. On a platform with has 64 bit ints, there will
- * be many ints which are outside the range supported by NC_INT. But
- * since NC_INT is an external format, it has to mean the same thing
- * everywhere. */
+/*! Max or min values for a type. Nothing greater/smaller can be
+ * stored in a netCDF file for their associated types. Recall that a C
+ * compiler may define int to be any length it wants, but a NC_INT is
+ * *always* a 4 byte signed int. On a platform with 64 bit ints,
+ * there will be many ints which are outside the range supported by
+ * NC_INT. But since NC_INT is an external format, it has to mean the
+ * same thing everywhere. */
+/**@{*/
 #define NC_MAX_BYTE 127
 #define NC_MIN_BYTE (-NC_MAX_BYTE-1)
 #define NC_MAX_CHAR 255
@@ -121,221 +98,286 @@ extern "C" {
 #define X_INT64_MAX     (9223372036854775807LL)
 #define X_INT64_MIN     (-X_INT64_MAX - 1)
 #define X_UINT64_MAX    (18446744073709551615ULL)
+/**@}*/
 
-/*
- * The above values are defaults.  If you wish a variable to use a
+/** Name of fill value attribute.  If you wish a variable to use a
  * different value than the above defaults, create an attribute with
- * the same type as the variable and the following reserved name. The
- * value you give the attribute will be used as the fill value for
- * that variable.
- */
+ * the same type as the variable and this reserved name. The value you
+ * give the attribute will be used as the fill value for that
+ * variable. */
 #define _FillValue	"_FillValue"
-#define NC_FILL		0	/* argument to ncsetfill to clear NC_NOFILL */
-#define NC_NOFILL	0x100	/* Don't fill data section an records */
+#define NC_FILL		0	/**< Argument to nc_set_fill() to clear NC_NOFILL */
+#define NC_NOFILL	0x100	/**< Argument to nc_set_fill() to turn off filling of data. */
 
-/*
- * Use these 'mode' flags for nc_open.
- */
-#define NC_NOWRITE	0	/* default is read only */
-#define NC_WRITE    	0x0001	/* read & write */
+/* Define the ioflags bits for nc_create and nc_open.
+   currently unused: 0x0010,0x0020,0x0040,0x0080
+   and the whole upper 16 bits
+*/
 
-/*
- * Use these 'mode' flags for nc_create.
- */
-#define NC_CLOBBER	0
-#define NC_NOCLOBBER	0x0004	/* Don't destroy existing file on create */
-#define NC_64BIT_OFFSET 0x0200  /* Use large (64-bit) file offsets */
-#define NC_NETCDF4      0x1000  /* Use netCDF-4/HDF5 format */
-#define NC_CLASSIC_MODEL 0x0100 /* Enforce classic model when used with NC_NETCDF4. */
+#define NC_NOWRITE	 0x0000	/**< Set read-only access for nc_open(). */
+#define NC_WRITE    	 0x0001	/**< Set read-write access for nc_open(). */
+/* unused: 0x0002 */
+#define NC_CLOBBER	 0x0000 /**< Destroy existing file. Mode flag for nc_create(). */
+#define NC_NOCLOBBER     0x0004	/**< Don't destroy existing file. Mode flag for nc_create(). */
 
-/*
- * Use these 'mode' flags for both nc_create and nc_open.
- */
-#define NC_SHARE       0x0800	/* Share updates, limit cacheing */
-#define NC_MPIIO       0x2000 
-#define NC_MPIPOSIX    0x4000
-#define NC_PNETCDF     0x8000
+#define NC_DISKLESS      0x0008  /**< Use diskless file. Mode flag for nc_open() or nc_create(). */
+#define NC_MMAP          0x0010  /**< Use diskless file with mmap. Mode flag for nc_open() or nc_create(). */
 
-/* The following flag currently is ignored, but use in
+#define NC_CLASSIC_MODEL 0x0100 /**< Enforce classic model. Mode flag for nc_create(). */
+#define NC_64BIT_OFFSET  0x0200  /**< Use large (64-bit) file offsets. Mode flag for nc_create(). */
+
+/** \deprecated The following flag currently is ignored, but use in
  * nc_open() or nc_create() may someday support use of advisory
- * locking to prevent multiple writers from clobbering a file 
+ * locking to prevent multiple writers from clobbering a file
  */
-#define NC_LOCK		0x0400	/* Use locking if available */
+#define NC_LOCK          0x0400	
 
-/*
- * Starting with version 3.6, there are different format netCDF
- * files. 4.0 introduces the third one. These defines are only for
- * the nc_set_default_format function.
+/** Share updates, limit cacheing.
+Use this in mode flags for both nc_create() and nc_open(). */
+#define NC_SHARE         0x0800	
+
+#define NC_NETCDF4       0x1000  /**< Use netCDF-4/HDF5 format. Mode flag for nc_create(). */
+
+/** Turn on MPI I/O.
+Use this in mode flags for both nc_create() and nc_open(). */
+#define NC_MPIIO         0x2000 
+/** Turn on MPI POSIX I/O.
+Use this in mode flags for both nc_create() and nc_open(). */
+#define NC_MPIPOSIX      0x4000
+#define NC_PNETCDF       0x8000	/**< Use parallel-netcdf library. Mode flag for nc_open(). */
+
+/** Format specifier for nc_set_default_format().  Starting with
+ * version 3.6, there are different format netCDF files. 4.0
+ * introduces the third one. \see netcdf_format
  */
+/**@{*/
 #define NC_FORMAT_CLASSIC (1)
 #define NC_FORMAT_64BIT   (2)
 #define NC_FORMAT_NETCDF4 (3)
-#define NC_FORMAT_NETCDF4_CLASSIC  (4) /* create netcdf-4 files, with NC_CLASSIC_MODEL. */
+#define NC_FORMAT_NETCDF4_CLASSIC  (4)
+/**@}*/
 
-/*
- * Let nc__create() or nc__open() figure out
- * as suitable chunk size.
- */
+/** Let nc__create() or nc__open() figure out as suitable chunk
+ * size. */
 #define NC_SIZEHINT_DEFAULT 0
 
-/*
- * In nc__enddef(), align to the chunk size.
- */
+/** In nc__enddef(), align to the chunk size. */
 #define NC_ALIGN_CHUNK ((size_t)(-1))
 
-/*
- * 'size' argument to ncdimdef for an unlimited dimension
- */
+/** Size argument to nc_def_dim() for an unlimited dimension. */
 #define NC_UNLIMITED 0L
 
-/*
- * attribute id to put/get a global attribute
- */
+/** Attribute id to put/get a global attribute. */
 #define NC_GLOBAL -1
 
-/*
- * These maximums are enforced by the interface, to facilitate writing
- * applications and utilities.  However, nothing is statically allocated to
- * these sizes internally.
- */
-#define NC_MAX_DIMS	1024	 /* max dimensions per file */
-#define NC_MAX_ATTRS	8192	 /* max global or per variable attributes */
-#define NC_MAX_VARS	8192	 /* max variables per file */
-#define NC_MAX_NAME	256	 /* max length of a name */
-/* As a rule, NC_MAX_VAR_DIMS <= NC_MAX_DIMS*/
-#define NC_MAX_VAR_DIMS	1024 /* max per variable dimensions */
+/** 
+Maximum for classic library.
 
-/* This is the max size of an SD dataset name in HDF4. */
-#define NC_MAX_HDF4_NAME 64 /* From HDF4 documentation. */
+In the classic netCDF model there are maximum values for the number of
+dimensions in the file (\ref NC_MAX_DIMS), the number of global or per
+variable attributes (\ref NC_MAX_ATTRS), the number of variables in
+the file (\ref NC_MAX_VARS), and the length of a name (\ref
+NC_MAX_NAME).
 
-/* In HDF5 files you can set the endianness of variables with
- * nc_def_var_endian(). These defines are used there. */   
+This maximum is enforced by the interface, to facilitate writing
+applications and utilities.  However, nothing is statically allocated
+to this size internally.
+
+This maximum is not used for netCDF-4/HDF5 files unless they were
+created with the ::NC_CLASSIC_MODEL flag.
+
+As a rule, NC_MAX_VAR_DIMS <= NC_MAX_DIMS.
+*/
+/**@{*/
+#define NC_MAX_DIMS	1024	
+#define NC_MAX_ATTRS	8192	
+#define NC_MAX_VARS	8192	
+#define NC_MAX_NAME	256	
+#define NC_MAX_VAR_DIMS	1024 /**< max per variable dimensions */
+/**@}*/
+
+/** This is the max size of an SD dataset name in HDF4 (from HDF4 documentation).*/
+#define NC_MAX_HDF4_NAME 64 
+
+/** In HDF5 files you can set the endianness of variables with
+    nc_def_var_endian(). This define is used there. */   
+/**@{*/
 #define NC_ENDIAN_NATIVE 0
 #define NC_ENDIAN_LITTLE 1
 #define NC_ENDIAN_BIG    2
+/**@}*/
 
-/* In HDF5 files you can set storage for each variable to be either
- * contiguous or chunked, with nc_def_var_chunking().  These defines
- * are used there. */
+/** In HDF5 files you can set storage for each variable to be either
+ * contiguous or chunked, with nc_def_var_chunking().  This define is
+ * used there. */
+/**@{*/
 #define NC_CHUNKED    0
 #define NC_CONTIGUOUS 1
+/**@}*/
 
-/* In HDF5 files you can set check-summing for each variable.
- * Currently the only checksum available is Fletcher-32, which can be
- * set with the function nc_def_var_fletcher32.  These defines are used
- * there. */
+/** In HDF5 files you can set check-summing for each variable.
+Currently the only checksum available is Fletcher-32, which can be set
+with the function nc_def_var_fletcher32.  These defines are used
+there. */
+/**@{*/
 #define NC_NOCHECKSUM 0
 #define NC_FLETCHER32 1
+/**@}*/
 
-/* In HDF5 files you can specify that a shuffle filter should be used
- * on each chunk of a variable to improve compression for that
- * variable.  This per-variable shuffle property can be set with the
- * function nc_def_var_deflate.  These defines are used there. */
+/**@{*/
+/** Control the HDF5 shuffle filter. In HDF5 files you can specify
+ * that a shuffle filter should be used on each chunk of a variable to
+ * improve compression for that variable. This per-variable shuffle
+ * property can be set with the function nc_def_var_deflate(). */
 #define NC_NOSHUFFLE 0
 #define NC_SHUFFLE   1
+/**@}*/
 
-/*
- * The netcdf version 3 functions all return integer error status.
- * These are the possible values, in addition to certain
- * values from the system errno.h.
+/** The netcdf version 3 functions all return integer error status.
+ * These are the possible values, in addition to certain values from
+ * the system errno.h.
  */
-
 #define NC_ISSYSERR(err)	((err) > 0)
 
-#define	NC_NOERR	0	/* No Error */
+#define	NC_NOERR	0   /**< No Error */
+#define NC2_ERR         (-1)       /**< Returned for all errors in the v2 API. */
 
-#define NC2_ERR         (-1)    /* Returned for all errors in the v2 API. */
-#define	NC_EBADID	(-33)	/* Not a netcdf id */
-#define	NC_ENFILE	(-34)	/* Too many netcdfs open */
-#define	NC_EEXIST	(-35)	/* netcdf file exists && NC_NOCLOBBER */
-#define	NC_EINVAL	(-36)	/* Invalid Argument */
-#define	NC_EPERM	(-37)	/* Write to read only */
-#define NC_ENOTINDEFINE	(-38)	/* Operation not allowed in data mode */
-#define	NC_EINDEFINE	(-39)	/* Operation not allowed in define mode */
-#define	NC_EINVALCOORDS	(-40)	/* Index exceeds dimension bound */
-#define	NC_EMAXDIMS	(-41)	/* NC_MAX_DIMS exceeded */
-#define	NC_ENAMEINUSE	(-42)	/* String match to name in use */
-#define NC_ENOTATT	(-43)	/* Attribute not found */
-#define	NC_EMAXATTS	(-44)	/* NC_MAX_ATTRS exceeded */
-#define NC_EBADTYPE	(-45)	/* Not a netcdf data type */
-#define NC_EBADDIM	(-46)	/* Invalid dimension id or name */
-#define NC_EUNLIMPOS	(-47)	/* NC_UNLIMITED in the wrong index */
-#define	NC_EMAXVARS	(-48)	/* NC_MAX_VARS exceeded */
-#define NC_ENOTVAR	(-49)	/* Variable not found */
-#define NC_EGLOBAL	(-50)	/* Action prohibited on NC_GLOBAL varid */
-#define NC_ENOTNC	(-51)	/* Not a netcdf file */
-#define NC_ESTS        	(-52)	/* In Fortran, string too short */
-#define NC_EMAXNAME    	(-53)	/* NC_MAX_NAME exceeded */
-#define NC_EUNLIMIT    	(-54)	/* NC_UNLIMITED size already in use */
-#define NC_ENORECVARS  	(-55)	/* nc_rec op when there are no record vars */
-#define NC_ECHAR	(-56)	/* Attempt to convert between text & numbers */
-#define NC_EEDGE	(-57)	/* Start+count exceeds dimension bound */
-#define NC_ESTRIDE	(-58)	/* Illegal stride */
-#define NC_EBADNAME	(-59)	/* Attribute or variable name
-                                         contains illegal characters */
+/** Not a netcdf id.
+
+The specified netCDF ID does not refer to an
+open netCDF dataset. */
+#define	NC_EBADID	(-33)	   
+#define	NC_ENFILE	(-34)	   /**< Too many netcdfs open */
+#define	NC_EEXIST	(-35)	   /**< netcdf file exists && NC_NOCLOBBER */
+#define	NC_EINVAL	(-36)	   /**< Invalid Argument */
+#define	NC_EPERM	(-37)	   /**< Write to read only */
+
+/** Operation not allowed in data mode. This is returned for netCDF
+classic or 64-bit offset files, or for netCDF-4 files, when they were
+been created with ::NC_CLASSIC_MODEL flag in nc_create(). */
+#define NC_ENOTINDEFINE	(-38)	   
+
+/** Operation not allowed in define mode.
+
+The specified netCDF is in define mode rather than data mode. 
+
+With netCDF-4/HDF5 files, this error will not occur, unless
+::NC_CLASSIC_MODEL was used in nc_create().
+ */
+#define	NC_EINDEFINE	(-39)	   
+
+/** Index exceeds dimension bound.
+
+The specified corner indices were out of range for the rank of the
+specified variable. For example, a negative index or an index that is
+larger than the corresponding dimension length will cause an error. */
+#define	NC_EINVALCOORDS	(-40)	   
+#define	NC_EMAXDIMS	(-41)	   /**< NC_MAX_DIMS exceeded */
+#define	NC_ENAMEINUSE	(-42)	   /**< String match to name in use */
+#define NC_ENOTATT	(-43)	   /**< Attribute not found */
+#define	NC_EMAXATTS	(-44)	   /**< NC_MAX_ATTRS exceeded */
+#define NC_EBADTYPE	(-45)	   /**< Not a netcdf data type */
+#define NC_EBADDIM	(-46)	   /**< Invalid dimension id or name */
+#define NC_EUNLIMPOS	(-47)	   /**< NC_UNLIMITED in the wrong index */
+
+/** NC_MAX_VARS exceeded. Max number of variables exceeded in a
+classic or 64-bit offset file, or an netCDF-4 file with
+::NC_CLASSIC_MODEL on. */
+#define	NC_EMAXVARS	(-48)	   
+
+/** Variable not found.
+
+The variable ID is invalid for the specified netCDF dataset. */
+#define NC_ENOTVAR	(-49)	   
+#define NC_EGLOBAL	(-50)	   /**< Action prohibited on NC_GLOBAL varid */
+#define NC_ENOTNC	(-51)	   /**< Not a netcdf file */
+#define NC_ESTS        	(-52)	   /**< In Fortran, string too short */
+#define NC_EMAXNAME    	(-53)	   /**< NC_MAX_NAME exceeded */
+#define NC_EUNLIMIT    	(-54)	   /**< NC_UNLIMITED size already in use */
+#define NC_ENORECVARS  	(-55)	   /**< nc_rec op when there are no record vars */
+#define NC_ECHAR	(-56)	   /**< Attempt to convert between text & numbers */
+
+/** Start+count exceeds dimension bound.
+
+The specified edge lengths added to the specified corner would have
+referenced data out of range for the rank of the specified
+variable. For example, an edge length that is larger than the
+corresponding dimension length minus the corner index will cause an
+error. */
+#define NC_EEDGE	(-57)	   
+#define NC_ESTRIDE	(-58)	   /**< Illegal stride */
+#define NC_EBADNAME	(-59)	   /**< Attribute or variable name contains illegal characters */
 /* N.B. following must match value in ncx.h */
-#define NC_ERANGE	(-60)	/* Math result not representable */
-#define NC_ENOMEM	(-61)	/* Memory allocation (malloc) failure */
 
-#define NC_EVARSIZE     (-62)   /* One or more variable sizes violate
-				   format constraints */ 
-#define NC_EDIMSIZE     (-63)   /* Invalid dimension size */
-#define NC_ETRUNC       (-64)   /* File likely truncated or possibly corrupted */
+/** Math result not representable.
 
-#define NC_EAXISTYPE    (-65)   /* Unknown axis type. */
+One or more of the values are out of the range of values representable
+by the desired type. */
+#define NC_ERANGE	(-60)	   
+#define NC_ENOMEM	(-61)	   /**< Memory allocation (malloc) failure */
+#define NC_EVARSIZE     (-62)      /**< One or more variable sizes violate format constraints */ 
+#define NC_EDIMSIZE     (-63)      /**< Invalid dimension size */
+#define NC_ETRUNC       (-64)      /**< File likely truncated or possibly corrupted */
+#define NC_EAXISTYPE    (-65)      /**< Unknown axis type. */
 
 /* Following errors are added for DAP */
-#define NC_EDAP         (-66)   /* Generic DAP error */
-#define NC_ECURL        (-67)   /* Generic libcurl error */
-#define NC_EIO          (-68)   /* Generic IO error */
-#define NC_ENODATA      (-69)   /* Attempt to access variable with no data */
-#define NC_EDAPSVC      (-70)   /* DAP Server side error */
-#define NC_EDAS		(-71)   /* Malformed or inaccessible DAS */
-#define NC_EDDS		(-72)   /* Malformed or inaccessible DDS */
-#define NC_EDATADDS	(-73)   /* Malformed or inaccessible DATADDS */
-#define NC_EDAPURL	(-74)   /* Malformed DAP URL */
-#define NC_EDAPCONSTRAINT (-75)   /* Malformed DAP Constraint*/
-#define NC_ETRANSLATION (-76)   /* Untranslatable construct */
+#define NC_EDAP         (-66)      /**< Generic DAP error */
+#define NC_ECURL        (-67)      /**< Generic libcurl error */
+#define NC_EIO          (-68)      /**< Generic IO error */
+#define NC_ENODATA      (-69)      /**< Attempt to access variable with no data */
+#define NC_EDAPSVC      (-70)      /**< DAP server error */
+#define NC_EDAS		(-71)      /**< Malformed or inaccessible DAS */
+#define NC_EDDS		(-72)      /**< Malformed or inaccessible DDS */
+#define NC_EDATADDS	(-73)      /**< Malformed or inaccessible DATADDS */
+#define NC_EDAPURL	(-74)      /**< Malformed DAP URL */
+#define NC_EDAPCONSTRAINT (-75)      /**< Malformed DAP Constraint*/
+#define NC_ETRANSLATION (-76)      /**< Untranslatable construct */
 
 /* The following was added in support of netcdf-4. Make all netcdf-4
    error codes < -100 so that errors can be added to netcdf-3 if
    needed. */
 #define NC4_FIRST_ERROR  (-100)
-#define NC_EHDFERR       (-101) /* Error at HDF5 layer. */
-#define NC_ECANTREAD     (-102) /* Can't read. */
-#define NC_ECANTWRITE    (-103) /* Can't write. */
-#define NC_ECANTCREATE   (-104) /* Can't create. */
-#define NC_EFILEMETA     (-105) /* Problem with file metadata. */
-#define NC_EDIMMETA      (-106) /* Problem with dimension metadata. */
-#define NC_EATTMETA      (-107) /* Problem with attribute metadata. */
-#define NC_EVARMETA      (-108) /* Problem with variable metadata. */
-#define NC_ENOCOMPOUND   (-109) /* Not a compound type. */
-#define NC_EATTEXISTS    (-110) /* Attribute already exists. */
-#define NC_ENOTNC4       (-111) /* Attempting netcdf-4 operation on netcdf-3 file. */  
-#define NC_ESTRICTNC3    (-112) /* Attempting netcdf-4 operation on strict nc3 netcdf-4 file. */  
-#define NC_ENOTNC3       (-113) /* Attempting netcdf-3 operation on netcdf-4 file. */  
-#define NC_ENOPAR        (-114) /* Parallel operation on file opened for non-parallel access. */  
-#define NC_EPARINIT      (-115) /* Error initializing for parallel access. */  
-#define NC_EBADGRPID     (-116) /* Bad group ID. */  
-#define NC_EBADTYPID     (-117) /* Bad type ID. */  
-#define NC_ETYPDEFINED   (-118) /* Type has already been defined and may not be edited. */
-#define NC_EBADFIELD     (-119) /* Bad field ID. */  
-#define NC_EBADCLASS     (-120) /* Bad class. */  
-#define NC_EMAPTYPE      (-121) /* Mapped access for atomic types only. */  
-#define NC_ELATEFILL     (-122) /* Attempt to define fill value when data already exists. */
-#define NC_ELATEDEF      (-123) /* Attempt to define var properties, like deflate, after enddef. */
-#define NC_EDIMSCALE     (-124) /* Probem with HDF5 dimscales. */
-#define NC_ENOGRP        (-125) /* No group found. */
-#define NC_ESTORAGE      (-126) /* Can't specify both contiguous and chunking. */
-#define NC_EBADCHUNK     (-127) /* Bad chunksize. */
-#define NC_ENOTBUILT     (-128) /* Attempt to use feature that was not turned on when netCDF was built. */
-#define NC4_LAST_ERROR   (-128) 
+
+/** Error at HDF5 layer. */
+#define NC_EHDFERR       (-101)    
+#define NC_ECANTREAD     (-102)    /**< Can't read. */
+#define NC_ECANTWRITE    (-103)    /**< Can't write. */
+#define NC_ECANTCREATE   (-104)    /**< Can't create. */
+#define NC_EFILEMETA     (-105)    /**< Problem with file metadata. */
+#define NC_EDIMMETA      (-106)    /**< Problem with dimension metadata. */
+#define NC_EATTMETA      (-107)    /**< Problem with attribute metadata. */
+#define NC_EVARMETA      (-108)    /**< Problem with variable metadata. */
+#define NC_ENOCOMPOUND   (-109)    /**< Not a compound type. */
+#define NC_EATTEXISTS    (-110)    /**< Attribute already exists. */
+#define NC_ENOTNC4       (-111)    /**< Attempting netcdf-4 operation on netcdf-3 file. */  
+
+/** Attempting netcdf-4 operation on strict nc3 netcdf-4 file. */  
+#define NC_ESTRICTNC3    (-112)    
+#define NC_ENOTNC3       (-113)    /**< Attempting netcdf-3 operation on netcdf-4 file. */  
+#define NC_ENOPAR        (-114)    /**< Parallel operation on file opened for non-parallel access. */  
+#define NC_EPARINIT      (-115)    /**< Error initializing for parallel access. */  
+#define NC_EBADGRPID     (-116)    /**< Bad group ID. */  
+#define NC_EBADTYPID     (-117)    /**< Bad type ID. */  
+#define NC_ETYPDEFINED   (-118)    /**< Type has already been defined and may not be edited. */
+#define NC_EBADFIELD     (-119)    /**< Bad field ID. */  
+#define NC_EBADCLASS     (-120)    /**< Bad class. */  
+#define NC_EMAPTYPE      (-121)    /**< Mapped access for atomic types only. */  
+#define NC_ELATEFILL     (-122)    /**< Attempt to define fill value when data already exists. */
+#define NC_ELATEDEF      (-123)    /**< Attempt to define var properties, like deflate, after enddef. */
+#define NC_EDIMSCALE     (-124)    /**< Probem with HDF5 dimscales. */
+#define NC_ENOGRP        (-125)    /**< No group found. */
+#define NC_ESTORAGE      (-126)    /**< Can't specify both contiguous and chunking. */
+#define NC_EBADCHUNK     (-127)    /**< Bad chunksize. */
+#define NC_ENOTBUILT     (-128)    /**< Attempt to use feature that was not turned on when netCDF was built. */
+#define NC_EDISKLESS     (-129)    /**< Error in using diskless  access. */ 
+
+#define NC4_LAST_ERROR   (-129) 
 
 /* This is used in netCDF-4 files for dimensions without coordinate
  * vars. */
 #define DIM_WITHOUT_VARIABLE "This is a netCDF dimension but not a netCDF variable."
 
-/* This is here at the request of the NCO team to support the stupid
+/* This is here at the request of the NCO team to support our
  * mistake of having chunksizes be first ints, then size_t. Doh! */
 #define NC_HAVE_NEW_CHUNKING_API 1
 
@@ -542,16 +584,16 @@ EXTERNL int
 nc_inq_compound_fielddim_sizes(int ncid, nc_type xtype, int fieldid, 
 			       int *dim_sizes);
 
-/* This is the type of arrays of vlens. */
+/** This is the type of arrays of vlens. */
 typedef struct {
-    size_t len; /* Length of VL data (in base type units) */
-    void *p;    /* Pointer to VL data */
+    size_t len; /**< Length of VL data (in base type units) */
+    void *p;    /**< Pointer to VL data */
 } nc_vlen_t;
 
-/* This is used when creating a compound type. It calls a mysterious C
- * macro which was found carved into one of the blocks of the
- * Newgrange passage tomb in County Meath, Ireland. This code has been
- * carbon dated to 3200 B.C.E. */
+/** Calculate an offset for creating a compound type. This calls a
+ * mysterious C macro which was found carved into one of the blocks of
+ * the Newgrange passage tomb in County Meath, Ireland. This code has
+ * been carbon dated to 3200 B.C.E. */
 #define NC_COMPOUND_OFFSET(S,M)    (offsetof(S,M))
 
 /* Create a variable length type. */
@@ -737,7 +779,7 @@ nc_def_var_fill(int ncid, int varid, int no_fill, const void *fill_value);
 
 /* Inq fill value setting for a var. */
 EXTERNL int
-nc_inq_var_fill(int ncid, int varid, int *no_fill, void *fill_value);
+nc_inq_var_fill(int ncid, int varid, int *no_fill, void *fill_valuep);
 
 /* Define the endianness of a variable. */
 EXTERNL int
@@ -1100,16 +1142,16 @@ nc_get_var1_string(int ncid, int varid, const size_t *indexp,
 /* Begin {put,get}_vara */
 
 EXTERNL int
-nc_put_vara_text(int ncid, int varid,
-	const size_t *startp, const size_t *countp, const char *op);
+nc_put_vara_text(int ncid, int varid, const size_t *startp, 
+		 const size_t *countp, const char *op);
 
 EXTERNL int
-nc_get_vara_text(int ncid, int varid,
-	const size_t *startp, const size_t *countp, char *ip);
+nc_get_vara_text(int ncid, int varid, const size_t *startp, 
+		 const size_t *countp, char *ip);
 
 EXTERNL int
-nc_put_vara_uchar(int ncid, int varid,
-	const size_t *startp, const size_t *countp, const unsigned char *op);
+nc_put_vara_uchar(int ncid, int varid, const size_t *startp, 
+		  const size_t *countp, const unsigned char *op);
 
 EXTERNL int
 nc_get_vara_uchar(int ncid, int varid, const size_t *startp, 
@@ -1662,9 +1704,8 @@ nctypelen(nc_type datatype);
  */
 #ifndef NO_NETCDF_2
 
-/*
- * Backward compatible aliases
- */
+/** Backward compatible alias. */
+/**@{*/
 #define FILL_BYTE	NC_FILL_BYTE
 #define FILL_CHAR	NC_FILL_CHAR
 #define FILL_SHORT	NC_FILL_SHORT
@@ -1677,6 +1718,7 @@ nctypelen(nc_type datatype);
 #define MAX_NC_VARS	NC_MAX_VARS
 #define MAX_NC_NAME	NC_MAX_NAME
 #define MAX_VAR_DIMS	NC_MAX_VAR_DIMS
+/**@}*/
 
 
 /*
@@ -1830,4 +1872,12 @@ ncrecput(int ncid, long recnum, void *const *datap);
 }
 #endif
 
+/* Temporary hack to shut up warnings */
+#ifndef __MINGW32_VERSION 
+#define END_OF_MAIN()
+#endif
+
 #endif /* _NETCDF_ */
+
+
+
