@@ -27,21 +27,6 @@ and the per-retrieval maximum size
 /* Max number of cache nodes */
 #define DFALTCACHECOUNT (100)
 
-/* Define a tracker for memory to support*/
-/* the concatenation*/
-
-struct NCMEMORY {
-    void* memory;
-    char* next; /* where to store the next chunk of data*/
-}; 
-
-typedef int nc_tactic;
-#define tactic_null	0
-#define tactic_all	1
-#define tactic_partial	2
-#define tactic_grid	4
-#define tactic_var	8
-
 typedef struct Getvara {
     void* memory; /* where result is put*/
     struct NCcachenode* cache;
@@ -49,7 +34,7 @@ typedef struct Getvara {
     /* associated variable*/
     OCtype dsttype;
     CDFnode* target;
-    CDFnode* target0;
+    int wholevariable;
 } Getvara;
 
 #endif /*GETVARA_H*/
