@@ -66,7 +66,7 @@ rel_v1hs(v1hs *gsp)
 	int status;
 	if(gsp->offset == OFF_NONE || gsp->base == NULL)
 		return ENOERR;
-	status = gsp->nciop->rel(gsp->nciop, gsp->offset,
+	status = ncio_rel(gsp->nciop, gsp->offset,
 			 gsp->flags == RGN_WRITE ? RGN_MODIFIED : 0);
 	gsp->end = NULL;
 	gsp->pos = NULL;
@@ -96,7 +96,7 @@ fault_v1hs(v1hs *gsp, size_t extent)
 	if(extent > gsp->extent)
 		gsp->extent = extent;	
 
-	status = gsp->nciop->get(gsp->nciop,
+	status = ncio_get(gsp->nciop,
 		 	gsp->offset, gsp->extent,
 			gsp->flags, &gsp->base);
 	if(status)
@@ -1117,7 +1117,10 @@ NC_computeshapes(NC *ncp)
 		{
 	  		if(first_rec == NULL)	
 				first_rec = *vpp;
-			ncp->recsize += (*vpp)->len;
+			if((*vpp)->len == UINT32_MAX)
+			    ncp->recsize += (*vpp)->dsizes[0];
+			else
+			    ncp->recsize += (*vpp)->len;
 		}
 		else
 		{
@@ -1211,9 +1214,7 @@ ncx_put_NC(const NC *ncp, void **xpp, off_t offset, size_t extent)
 				extent = 4096;
 		}
 		else if(extent > ncp->chunk)
-		{
-			extent = ncp->chunk;
-		}
+		    extent = ncp->chunk;
 		
 		ps.offset = 0;
 		ps.extent = extent;
@@ -1315,15 +1316,13 @@ nc_get_NC(NC *ncp)
 			        extent = filesize;
 		}
 		else if(extent > ncp->chunk)
-		{
-			extent = ncp->chunk;
-		}
+		    extent = ncp->chunk;
 
 		/*
 		 * Invalidate the I/O buffers to force a read of the header
 		 * region.
 		 */
-		status = gs.nciop->sync(gs.nciop);
+		status = ncio_sync(gs.nciop);
 		if(status)
 			return status;
 
